// Copyright 2017 SiegeLord
//
// See LICENSE for terms.

use allegro_audio::{AudioAddon, Sample};
use std::collections::HashMap;
use std::rc::Rc;

pub struct SampleCache
{
	samples: HashMap<String, Rc<Sample>>,
}

impl SampleCache
{
	pub fn new() -> SampleCache
	{
		SampleCache
		{
			samples: HashMap::new(),
		}
	}
	
	pub fn load(&mut self, audio: &AudioAddon, filename: &str) -> Result<Rc<Sample>, String>
	{
		if let Some(bmp) = self.samples.get(filename)
		{
			return Ok(bmp.clone());
		}
		
		let bmp = Rc::new(try!(Sample::load(audio, filename).map_err(|_| format!("Could not load {}", filename))));
		self.samples.insert(filename.to_string(), bmp.clone());
		Ok(bmp)
	}
}
