// Copyright 2017 SiegeLord
// Licensed under GPL, see LICENSE for full terms

#![allow(dead_code)]

use allegro_audio::*;

pub struct Sfx
{
	persistent: Vec<SampleInstance>,
	temporary: Vec<SampleInstance>,
	sink: Sink,
	sound_mixer: Mixer,
	music_mixer: Mixer,
	music: Option<AudioStream>,
	do_music: bool,
}

impl Sfx
{
	pub fn new(addon: &AudioAddon, sound_volume: f32, music_volume: f32, do_music: bool) -> Sfx
	{
		let mut sink = Sink::new(addon).expect("Could not create audio sink.");
		let mut sound_mixer = Mixer::new(addon).unwrap();
		let mut music_mixer = Mixer::new(addon).unwrap();
		
		sound_mixer.set_gain(sound_volume).unwrap();
		music_mixer.set_gain(music_volume).unwrap();
		
		sound_mixer.attach(&mut sink).unwrap();
		music_mixer.attach(&mut sink).unwrap();
		Sfx
		{
			persistent: vec![],
			temporary: vec![],
			sink: sink,
			sound_mixer: sound_mixer,
			music_mixer: music_mixer,
			music: None,
			do_music: do_music,
		}
	}

	pub fn play(&mut self, sample: &Sample, addon: &AudioAddon)
	{
		let mut found_idx = self.temporary.len();
		for (idx, inst) in self.temporary.iter().enumerate()
		{
			if !inst.get_playing().unwrap_or(false)
			{
				found_idx = idx;
			}
		}
		if found_idx == self.temporary.len()
		{
			let mut inst = SampleInstance::new(addon).unwrap();
			inst.attach(&mut self.sound_mixer).unwrap();
			self.temporary.push(inst);
		}
		let inst = &mut self.temporary[found_idx];
		inst.set_sample(sample).unwrap();
		inst.set_playing(true).unwrap();
	}

	pub fn play_persistent(&mut self, sample: &Sample, addon: &AudioAddon) -> usize
	{
		let mut inst = SampleInstance::new(addon).unwrap();
		inst.set_sample(sample).unwrap();
		inst.attach(&mut self.sound_mixer).unwrap();
		inst.set_playing(true).unwrap();
		self.persistent.push(inst);
		self.persistent.len() - 1
	}

	pub fn get_instance(&mut self, inst_idx: usize) -> &mut SampleInstance
	{
		&mut self.persistent[inst_idx]
	}

	pub fn play_music(&mut self, name: &str, addon: &AudioAddon)
	{
		self.stop_music();
		if self.do_music
		{
			AudioStream::load(addon, name).map(|mut stream|
			{
				stream.attach(&mut self.music_mixer).unwrap();
				stream.set_gain(1.0).unwrap();
				stream.set_playmode(Playmode::Loop).unwrap();
				self.music = Some(stream);
			}).ok();
		}
	}

	pub fn stop_music(&mut self)
	{
		self.music = None;
	}
}
