// Copyright 2017 SiegeLord
// Licensed under GPL, see LICENSE for full terms

use std::io::prelude::*;
use std::fs::File;
use slr_config::*;
use std::path::Path;
use errors::*;
use std::cmp::Ordering;

pub fn load_config<T: ElementRepr>(file: &str, config: &mut T) -> Res<()>
{
	let mut contents = String::new();
	let mut f = File::open(file).map_err(|e| format!("{}: {}", e.to_string(), file))?;
	f.read_to_string(&mut contents).map_err(|e| format!("{}: {}", e.to_string(), file))?;
	
	let mut source = Source::new(Path::new(file), &contents);
	let element = ConfigElement::from_source(&mut source).map_err(|e| e.text)?;
	config.from_element(&element, Some(&source)).map_err(|e|
	{
		let mut text = String::new();
		for e in e
		{
			text.push_str(&e.text);
			text.push_str("\n");
		}
		text
	})?;
	Ok(())
}

pub fn min<T: PartialOrd>(a: T, b: T) -> T
{
	if let Some(order) = a.partial_cmp(&b)
	{
		match order
		{
			Ordering::Less => a,
			Ordering::Greater => b,
			Ordering::Equal => a,
		}
	}
	else
	{
		a
	}
}

pub fn max<T: PartialOrd>(a: T, b: T) -> T
{
	if let Some(order) = a.partial_cmp(&b)
	{
		match order
		{
			Ordering::Less => b,
			Ordering::Greater => a,
			Ordering::Equal => a,
		}
	}
	else
	{
		a
	}
}

pub fn l1_dist(x1: i32, y1: i32, x2: i32, y2: i32) -> i32
{
	(x2 - x1).abs() + (y2 - y1).abs()
}
